#!/bin/bash
set -euo pipefail #FIXME: exits on bad password

export TEXTDOMAIN=thunar-encryption-plugin
export TEXTDOMAINDIR="$(dirname $(dirname $0))/share/locale"
. gettext.sh

encryptedFolder="$1"
if [ -z "$encryptedFolder" ]; then
	echo "$(eval_gettext "usage: $0 path/to/folder")"
	exit 1
fi

if [ ! which cryfs 2>/dev/null ] && [ ! which encfs 2>/dev/null ]; then
	notify-send "GCryFolder" "$(eval_gettext "CryFS or EncFS not found.")" --icon=dialog-error
	exit 2
fi

if [[ "${encryptedFolder:0:1}" != / && "${encryptedFolder:0:2}" != ~[/a-z] ]]; then #if path is relative -> make it absolute
	encryptedFolder="$(pwd)/$encryptedFolder"
fi
encryptedName=$(basename "$encryptedFolder")
clearFolder="$HOME/GCryFolder/$encryptedName"

if [ ! -f "$encryptedFolder/cryfs.config" ] && [ ! -f "$encryptedFolder/.encfs6.xml" ]; then #clear folder - must be empty
	folderContent=$(ls -A "$encryptedFolder")
	if [ ! -z "$folderContent" ]; then
		notify-send "GCryFolder" "$(eval_gettext "Folder is not empty.")" --icon=dialog-error
		exit 3 #danger: folder is not empty
	fi
	
	pass=$(gcryfolder-password-prompt "$encryptedFolder")
	if [ -z "$pass" ]; then exit; fi
	#notify-send "GCryFolder" "$(eval_gettext "Decryption lasts a long time; please wait.")" --icon=dialog-information
	
	mkdir -p "$clearFolder"
	gio set -t stringv "$HOME/GCryFolder" metadata::emblems blueman-paired-emblem #key symbol ;-)
	gio set -t stringv  "$encryptedFolder" metadata::emblems blueman-paired-emblem
	if [ which cryfs 2>/dev/null ]; then #CryFS is better, so used by default if available
		export CRYFS_FRONTEND=noninteractive
		echo "$pass" | cryfs "$encryptedFolder" "$clearFolder"
	else
		echo -e "p\n$pass\n$pass" | encfs --stdinpass "$encryptedFolder" "$clearFolder"
	fi
	#notify-send "GCryFolder" "$(eval_gettext "Encrypted folder is ready.")" --icon=dialog-information
	xdg-open "$clearFolder"

else #already encrypted folder
	if mount | grep -q "on $clearFolder type fuse.cryfs" || mount | grep -q "on $clearFolder type fuse.encfs"; then
		fusermount -u "$clearFolder"
		rmdir "$clearFolder"
		if [ $(ls -A $(dirname "$clearFolder") | wc -w) == 0 ]; then rmdir $(dirname "$clearFolder"); fi #rmdir "$HOME/GCryFolder"
		#notify-send "GCryFolder" "$(eval_gettext "Encrypted folder is closed.")" --icon=dialog-information
	else
		pass=$(gcryfolder-password-prompt "$encryptedFolder")
		if [ -z "$pass" ]; then exit; fi
		#notify-send "GCryFolder" "$(eval_gettext "Decryption lasts a long time; please wait.")" --icon=dialog-information
		
		mkdir -p "$clearFolder"
		gio set -t stringv "$HOME/GCryFolder" metadata::emblems blueman-paired-emblem #key symbol ;-)
		if [ -f "$encryptedFolder/cryfs.config" ]; then
			export CRYFS_FRONTEND=noninteractive
			echo "$pass" | cryfs "$encryptedFolder" "$clearFolder" || true
		else
			echo "$pass" | encfs --stdinpass "$encryptedFolder" "$clearFolder" || true
		fi

		if ! mount | grep -q "on $clearFolder type fuse\."; then
			rmdir "$clearFolder"; rmdir $(dirname "$clearFolder") #rmdir "$HOME/GCryFolder"
			notify-send "GCryFolder" "$(eval_gettext "Bad password.")" --icon=dialog-error
		else
			#notify-send "GCryFolder" "$(eval_gettext "Encrypted folder is ready.")" --icon=dialog-information #xdg-open "$clearFolder"
			xdg-open "$clearFolder"
		fi
	fi
fi
